"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var _files, _finished, _numberOfFiles, _centralDirectory, _filesDataWritten;
Object.defineProperty(exports, "__esModule", { value: true });
const data_to_bytes_1 = require("./data-to-bytes");
const stream_1 = require("stream");
function int(n, length) {
    return Array.from({ length }, (k = n) => {
        n >>>= 8;
        return k & 0xff;
    });
}
class ZipStoreStream extends stream_1.Readable {
    constructor(files) {
        super();
        _files.set(this, void 0);
        _finished.set(this, false);
        _numberOfFiles.set(this, void 0);
        _centralDirectory.set(this, []);
        _filesDataWritten.set(this, 0);
        __classPrivateFieldSet(this, _files, files);
        __classPrivateFieldSet(this, _numberOfFiles, __classPrivateFieldGet(this, _files).length);
    }
    async _read() {
        // end if there is no files
        if (!__classPrivateFieldGet(this, _files).length) {
            if (!__classPrivateFieldGet(this, _finished)) {
                __classPrivateFieldSet(this, _finished, true);
                // writing central directory and finishing
                this.push(Buffer.from([
                    ...__classPrivateFieldGet(this, _centralDirectory),
                    0x50,
                    0x4b,
                    0x05,
                    0x06,
                    0x00,
                    0x00,
                    0x00,
                    0x00,
                    ...int(__classPrivateFieldGet(this, _numberOfFiles), 2),
                    ...int(__classPrivateFieldGet(this, _numberOfFiles), 2),
                    ...int(__classPrivateFieldGet(this, _centralDirectory).length, 4),
                    ...int(__classPrivateFieldGet(this, _filesDataWritten), 4),
                    0x00,
                    0x00,
                ]));
                // push the EOF-signaling `null` chunk.
                this.push(null);
            }
            return;
        }
        // getting next file to pipe
        const { path, data } = __classPrivateFieldGet(this, _files).shift();
        const { bytes, crc32 } = await data_to_bytes_1.dataToBuffer(data);
        // We support only ASCII encoded file names here
        const pathBytes = Buffer.from(path, 'ascii');
        const commonHeader = [
            0x0a,
            0x00,
            0x00,
            0x00,
            0x00,
            0x00,
            0x00,
            0x00,
            0x00,
            0x00,
            ...int(crc32, 4),
            ...int(bytes.length, 4),
            ...int(bytes.length, 4),
            ...int(pathBytes.length, 2),
            0x00,
            0x00,
        ];
        __classPrivateFieldGet(this, _centralDirectory).push(0x50, 0x4b, 0x01, 0x02, 0x14, 0x00, ...commonHeader, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, ...int(__classPrivateFieldGet(this, _filesDataWritten), 4), ...pathBytes);
        const fileData = [
            0x50,
            0x4b,
            0x03,
            0x04,
            ...commonHeader,
            ...pathBytes,
            ...bytes,
        ];
        __classPrivateFieldSet(this, _filesDataWritten, __classPrivateFieldGet(this, _filesDataWritten) + fileData.length);
        if (this.push(Buffer.from(fileData)))
            return this.read();
    }
}
exports.ZipStoreStream = ZipStoreStream;
_files = new WeakMap(), _finished = new WeakMap(), _numberOfFiles = new WeakMap(), _centralDirectory = new WeakMap(), _filesDataWritten = new WeakMap();
//# sourceMappingURL=index.js.map